/* -------------------------------------------------------------- */
/* Simple main program for EMM and POMME model evaluation         */
/*                                                                */
/* The program asks for time, location and magnetic indices.      */
/* In case the magnetic indices are unavailable, use their        */
/* default value.                                                 */
/* Contributions to the model can be turned on and off using      */
/* control parameters. These can also be used to set the          */
/* desired degree of the internal and external field.             */
/*                                          Stefan Maus, Oct-2009 */
/* Added freeing of memory and corrected minor bug in             */
/* mainfield_get_coeff_mp                   Stefan Maus, May-2010 */
/* -------------------------------------------------------------- */

#include <stdio.h>
#include <math.h>

#include "./modeldef.h"

#define MAGNETIC_MODEL       10      /* 1 = WMM-2005 for testing */
                                     /* 10 = EMM-2006 */
                                     /* 11 = EMM-2007 */
                                     /* 20 = POMME-4.0  (const secular acc, degree 36 */
                                     /* 21 = POMME-4.1  (const secular acc, degree 100 */
                                     /* 22 = POMME-4.2  (const secular acc, degree 720 */
                                     /* 25 = POMME-4.0s (split secular acc, degree 36 */
                                     /* 26 = POMME-4.1s (split secular acc, degree 100 */
                                     /* 27 = POMME-4.2s (split secular acc, degree 720 */
                                     /* 30 = POMME-5.0 (split secular acc, degree 60 */
                                     /* 35 = POMME-6.0 (split secular acc, degree 60 */
                                     /* 36 = POMME-6.1 (split secular acc, degree 130 */
                                     /* 37 = POMME-6.2 (split secular acc, degree 740 */

#if MAGNETIC_MODEL == 1
#define MAINFNAME       "./coeff/WMM-2005_EMM-format.cof"
#elif MAGNETIC_MODEL == 10
#define MAINFNAME       "./coeff/EMM-2006.cof"
#elif MAGNETIC_MODEL == 11
#define MAINFNAME       "./coeff/EMM-2007.cof"
#elif MAGNETIC_MODEL == 20
#define MAINFNAME       "./coeff/pomme-4.0.cof"
#elif MAGNETIC_MODEL == 21
#define MAINFNAME       "./coeff/pomme-4.1.cof"
#elif MAGNETIC_MODEL == 22
#define MAINFNAME       "./coeff/pomme-4.2.cof"
#elif MAGNETIC_MODEL == 25
#define MAINFNAME       "./coeff/pomme-4.0s.cof"
#elif MAGNETIC_MODEL == 26
#define MAINFNAME       "./coeff/pomme-4.1s.cof"
#elif MAGNETIC_MODEL == 27
#define MAINFNAME       "./coeff/pomme-4.2s.cof"
#elif MAGNETIC_MODEL == 30
#define MAINFNAME       "./coeff/pomme-5.0.cof"
#elif MAGNETIC_MODEL == 35
#define MAINFNAME       "./coeff/pomme-6.0.cof"
#elif MAGNETIC_MODEL == 36
#define MAINFNAME       "./coeff/pomme-6.1.cof"
#elif MAGNETIC_MODEL == 37
#define MAINFNAME       "./coeff/pomme-6.2.cof"
#endif

#define SM2GEOCF        "./coeff/sm2geo_coeff_3_6"
#define GSM2GEOCF       "./coeff/gsm2geo_coeff_3_6"

static tcontrol control;        /* array of user-control parameters */
static tmodel *model;           /* pointer to array containing model coefficients */

extern void mainfield_open_file_mp (char mainfname[], char sm2geo_fname[], char gsm2geo_fname[], int *control,
				   int ncontrol, tmodel **model,
				   int *model_nbytes);

extern void mainfield_geodetic_mp (int control[MAXCONTROL], tmodel *model,
                                   double fday, double geod_alt, double geod_lat, double lon,
                                   double est, double ist, double imf_by, double f107, double Em,
                                   double *geod_x, double *geod_y, double *geod_z, 
                                   double *h, double *f, double *incl, double *decl);

extern void mainfield_free_mp(tmodel **model);

static  double lon, geod_lat, geod_alt, fday;                      /* Position and time */
static  double E_st=0, I_st=0, IMF_By=0, f107=120.0, Em=0.5;       /* Magnetic indices for time varying external field */
static  double x, y, z,  h, f, incl, decl;                         /* Resulting field */

int main(void)
{
  int i, model_nbytes;

  printf("opening %s\n", MAINFNAME);
  mainfield_open_file_mp(MAINFNAME, SM2GEOCF, GSM2GEOCF, control, MAXCONTROL, &model, &model_nbytes);
  if (model_nbytes != sizeof(tmodel)) 
    {
      printf("Error: %d bytes were allocated for model, while %d should be needed\n",
             model_nbytes,sizeof(tmodel)); 
    }

#if MAGNETIC_MODEL == 1

  printf("\nMIMMIC_WMM mode:\n Result should agree with WMM-2005 to within about 0.2 nT in each magnetic component \n\n");

#else   /* mainfield_open_file_mp initializes the control parameters. You can now modify these: */

  control[POS_0] =         760;   /* default: model.deg_int, 0=off */
  control[POS_1] =         16;    /* default: model.deg_1st, 0=off */
  control[POS_2] =         16;    /* default: model.deg_2nd, 0=off */
  control[POS_3] =         16;    /* default: model.deg_2nd_plus, 0=off */

  control[POS_SM] =        (*model).deg_sm;       /* degree of SM field, default = (*model).deg_sm, 0=off */
  control[POS_SM_IND] =    (*model).deg_sm;       /* degree of field induced by SM field, default = (*model).deg_sm, 0=off */
  control[POS_F107] =      (*model).deg_f107;     /* degree of SM field, default = (*model).deg_sm, 0=off */
  control[POS_F107_IND] =  (*model).deg_f107;     /* degree of field induced by SM field, default = (*model).deg_sm, 0=off */
  control[POS_EST] =       1;                  /* Est dependent field: 1=on, 0=off */
  control[POS_IST] =       1;                  /* Ist dependent field: 1=on, 0=off */
  control[POS_EST_FAC] =   1;                  /* 1 = use model Est factor, 0 = use factor 1.0 */

  control[POS_GSM] =       (*model).deg_gsm;     /* degree of GSM field, default = (*model).deg_gsm, 0=off */
  control[POS_GSM_IND] =   (*model).deg_gsm;     /* degree of field induced by GSM field, default = (*model).deg_gsm, 0=off */
  control[POS_IMFBY] =     1;                 /* IMF-By dependent field: 1=on, 0=off */
  control[POS_IMFBY_IND] = 1;                 /* Induction by IMF-By dependent field: 1=on, 0=off */
  control[POS_EM] =        (*model).deg_Em;      /* degree of GSM field, default = (*model).deg_gsm, 0=off */
  control[POS_EM_IND] =    (*model).deg_Em;      /* degree of field induced by GSM field, default = (*model).deg_gsm, 0=off */


#endif

  printf("Universal time in floating point days since 00:00:00 on 1/1/2000? ");
  scanf("%lg%*[^\n]",&fday);
  printf("Longitude (degrees, positive to East)? ");
  scanf("%lg%*[^\n]",&lon);
  printf("Geodetic Latitude (degrees, positive to North)? ");
  scanf("%lg%*[^\n]",&geod_lat);
  printf("Altitude above mean sea level in km (WGS84 geoid)? ");
  scanf("%lg%*[^\n]",&geod_alt);

#if MAGNETIC_MODEL > 1
  printf("IMF-By 35 minutes earlier (enter '0' if unknown)? ");
  scanf("%lg%*[^\n]",&IMF_By);
  printf("Merging electric field, 60 minutes earlier (enter '0.5' if unknown)? ");
  scanf("%lg%*[^\n]",&Em);
  printf("E_st (enter '0' if unknown)? ");
  scanf("%lg%*[^\n]",&E_st);
  printf("I_st (enter '0' if unknown)? ");
  scanf("%lg%*[^\n]",&I_st);
  printf("F10.7, 81-day average, 20 months earlier (enter '120' if unknown)? ");
  scanf("%lg%*[^\n]",&f107);

#endif
  
  mainfield_geodetic_mp(control, model, fday, geod_alt, geod_lat, lon, E_st, I_st,
                        IMF_By, f107, Em, &x, &y, &z, &h, &f, &incl, &decl);                                         
  
  printf("\n\n");
  printf("    fday     lon     lat     alt    E_st    I_st   IMF_By     Em   F10.7-av\n");
  printf("----------------------------------------------------------------------------\n");
  printf("%8.1f %7.1f %7.1f %7.1f %7.1f %7.1f %7.1f %7.1f %9.1f\n\n",fday,lon,geod_lat,geod_alt,E_st,I_st,IMF_By,Em,f107);

  printf("        X          Y          Z          H          F       Incl       Decl\n");
  printf("----------------------------------------------------------------------------\n");
  printf("%10.2f %10.2f %10.2f %10.2f %10.2f %10.5f %10.5f\n\n",x,y,z,h,f,incl,decl);
  
  mainfield_free_mp(&model);
  
} /* End. */
